import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:ui';

import 'package:advertising_id/advertising_id.dart';
import 'package:device_info/device_info.dart';
import 'package:flutter/services.dart';
import 'package:http/http.dart' as http;
import 'package:intl/intl.dart';
import 'package:lemnisk_plugin/AttributeBuilder.dart';
import 'package:lemnisk_plugin/FlutterManifest.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:platform_device_id_platform_interface/platform_device_id_platform_interface.dart';
import 'package:uuid/uuid.dart';

import 'model/GenricModel.dart';

class LemniskHelper {
  static final DeviceInfoPlugin _deviceInfoPlugin = DeviceInfoPlugin();
  //get Device ID
  static Future<String?> get _getDeviceId async {
    String? deviceId;
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;
        deviceId = androidInfo.androidId;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceId = iosInfo.identifierForVendor;
      } else {
        deviceId = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceId = '';
    }
    return deviceId;
  }

  //get Plateform Device Type
  static Future<String?> get _getPlatFormDeviceType async {
    String? deviceType;
    try {
      if (Platform.isAndroid) {
        deviceType = "Android";
      } else if (Platform.isIOS) {
        deviceType = "IOS";
      }
    } on PlatformException {
      deviceType = '';
    }
    return deviceType;
  }

  //get Device Model Name
  static Future<String?> get _getDeviceModel async {
    String? deviceModel = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;
        deviceModel = androidInfo.model;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceModel = iosInfo.name;
      } else {
        deviceModel = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceModel = '';
    }
    return deviceModel;
  }

  //get Device Manufacture Model
  static Future<String?> get _getManufactureModel async {
    String? deviceManufacture = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;

        deviceManufacture = androidInfo.manufacturer;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceManufacture = iosInfo.systemName;
      } else {
        deviceManufacture = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceManufacture = '';
    }
    return deviceManufacture;
  }

  //get Device OS Version
  static Future<String?> get _getOSVersion async {
    String? deviceID = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await _deviceInfoPlugin.androidInfo;
        deviceID = androidInfo.version.release;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await _deviceInfoPlugin.iosInfo;
        deviceID = iosInfo.systemVersion;
      } else {
        deviceID = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceID = '';
    }
    return deviceID;
  }

  //get Device Google Advertising ID
  static Future<String?> get _getGoogleadvertisingID async {
    String? advertisingId;
    try {
      if (Platform.isAndroid) {
        try {
          advertisingId = await AdvertisingId.id(true);
        } on PlatformException {
          advertisingId = 'Failed to get platform version.';
        }
      } else if (Platform.isIOS) {
        advertisingId = await AdvertisingId.id(true);
      } else {
        advertisingId = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      advertisingId = '';
    }
    return advertisingId;
  }

  //get Device Google Advertising bool Value
  static Future<bool?> get _getGoogleadvertisingIDBoolValue async {
    bool? isLimitAdTrackingEnabled;
    try {
      if (Platform.isAndroid) {
        try {
          isLimitAdTrackingEnabled = await AdvertisingId.isLimitAdTrackingEnabled;
        } on PlatformException {
          isLimitAdTrackingEnabled = false;
        }
      } else if (Platform.isIOS) {
        isLimitAdTrackingEnabled = await AdvertisingId.isLimitAdTrackingEnabled;
      }
    } on PlatformException {
      isLimitAdTrackingEnabled = false;
    }
    return isLimitAdTrackingEnabled;
  }

  //get Device Width
  static Future<String?> get _getScreenWidth async {
    var physicalScreenSize = window.physicalSize;
    var physicalWidth = physicalScreenSize.width;
    return physicalWidth.toString();
  }

  //get Device Height
  static Future<String> get _getScreenHeight async {
    var physicalScreenSize = window.physicalSize;
    var physicalHeight = physicalScreenSize.height;
    return physicalHeight.toString();
  }

  //get App Name
  static Future<String> get _getAppName async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.appName;
  }

  //get App Version
  static Future<String> get _getAppVersion async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.version;
  }

  //get App Build No
  static Future<String> get _getBuildNumber async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.buildNumber;
  }

  //get Device Network IP
  static Future<String?> get _getNetworkIP async {
    final url = Uri.parse('https://api.ipify.org/');
    final response = await http.get(url);
    return response.body;
  }

  //get Payload Received Time
  static Future<dynamic> get _getReceivedAt async {
    return DateTime.now().toUtc().toString();
  }

  //get Device Payload Send Time
  static Future<String?> get _getSendAt async {
    DateTime getCurrentTimestamp = DateTime.now();
    String agreementSigned = formatISOTime(getCurrentTimestamp);
    return agreementSigned;
  }

  //get Device TimesTemp
  static Future<String?> get _getTimestemp async {
    DateTime getCurrentTimestamp = DateTime.now();
    String agreementSigned = formatISOTime(getCurrentTimestamp);
    return agreementSigned;
  }

  static String formatISOTime(DateTime date) {
    var duration = date.timeZoneOffset;
    if (duration.isNegative)
      return (DateFormat("yyyy-MM-ddTHH:mm:ss.mmm").format(date) + "-${duration.inHours.toString().padLeft(2, '0')}${(duration.inMinutes - (duration.inHours * 60)).toString().padLeft(2, '0')}");
    else
      return (DateFormat("yyyy-MM-ddTHH:mm:ss.mmm").format(date) + "+${duration.inHours.toString().padLeft(2, '0')}${(duration.inMinutes - (duration.inHours * 60)).toString().padLeft(2, '0')}");
  }

  //get Device Original TimesTemp
  static Future<String?> get _getOriginalTimestemp async {
    DateTime getCurrentTimestamp = DateTime.now();
    String agreementSigned = formatISOTime(getCurrentTimestamp);
    return agreementSigned;
  }

  //get Device Original TimesTemp
  static Future<String?> get _getServerTs async {
    return DateTime.now().toUtc().toString();
  }

  //get Device Original TimesTemp
  static Future<String?> get _getMessageId async {
    var uuid = Uuid();
    return uuid.v4();
  }

  static void track(String name, AttributeBuilder properties, OtherBuilder otherid) async {
    String serverUrl = FlutterManifest.SERVER_URL;
    String writeKey = FlutterManifest.WRITE_KEY;
    bool isDebugMode = FlutterManifest.DEBUG_MODE;
    bool isEnablePus = FlutterManifest.ENABLE_PUSH;

    GenricModel genricModel = GenricModel();
    genricModel.name = name;
    genricModel.properties = properties.properties;
    genricModel.otherid = otherid.otherid;
    GenricModel? fMain = await genricModel;
    //After taking data from user.. create our json payload
    //getting device details

    String? deviceId;
    String? deviceType;
    String? deviceManufacture;
    String? deviceModel;
    String? versionName;

    String? appName;
    String? appVersion;
    String? appBuild;

    String? googleAdvertisingID;
    dynamic? googleAdvertisingBool;

    String? networkIP;

    String? appScreenHeightWidth;
    String? appScreenHeightHeight;

    String? appDataReceivedAt;
    String? appDataSenddAt;
    String? appOriginalTimetemp;
    String? appTimestemp;
    String? appServerTs;
    String? appMsgId;

    try {
      deviceId = await _getDeviceId;
      deviceType = await _getPlatFormDeviceType;
      deviceModel = await _getDeviceModel;
      deviceManufacture = await _getManufactureModel;
      versionName = await _getOSVersion;
      appName = await _getAppName;
      appVersion = await _getAppVersion;
      appBuild = await _getBuildNumber;
      googleAdvertisingID = await _getGoogleadvertisingID;
      googleAdvertisingBool = await _getGoogleadvertisingIDBoolValue;
      networkIP = await _getNetworkIP;

      appScreenHeightHeight = await _getScreenHeight;
      appScreenHeightWidth = await _getScreenWidth;

      appDataReceivedAt = await _getReceivedAt;
      appDataSenddAt = await _getSendAt;

      appOriginalTimetemp = await _getOriginalTimestemp;
      appServerTs = await _getServerTs;
      appTimestemp = await _getTimestemp;
      appMsgId = await _getMessageId;
    } on PlatformException {
      deviceId = 'Failed to get information.';
    }

    Map<dynamic, dynamic> lib = {
      "library": {
        "name": "Flutter SDK",
        "version": "0.02",
      },
      "server_ts": appServerTs,
      "app": {
        "name": appName,
        "version": appVersion,
        "build": appBuild,
      },
      "device": {
        "id": deviceId,
        "advertisingId": googleAdvertisingID,
        "adTrackingEnabled": googleAdvertisingBool,
        "manufacturer": deviceManufacture,
        "model": deviceModel,
        "type": deviceType,
        "token": "ff15bc0c20c4aa6cd50854ff165fd265c838e5405bfeb9571066395b8c9da449",
      },
      "screen": {
        "width": appScreenHeightWidth,
        "height": appScreenHeightHeight,
        "density": 2,
      },
      "userAgent": {
        "osType": deviceType,
        "osVersion": versionName,
      },
      "ip": networkIP
    };

    Map map = {
      "id": deviceId,
      "messageId": appMsgId,
      "userId": "userId",
      "otherIds": fMain.otherid,
      "event": name,
      "properties": fMain.properties,
      "receivedAt": appDataReceivedAt,
      "sentAt": appDataSenddAt,
      "timestamp": appTimestemp,
      "type": "track",
      "originalTimestamp": appOriginalTimetemp,
      "writeKey": writeKey,
    };
    map.addAll({"context": lib});

    //After the full payload is created..now we send all to lemnisk server
    print('track event' + serverUrl);
    Uri uri = Uri.parse(serverUrl);
    var response = await http.post(uri, body: JsonEncoder().convert(map));
    if (response.statusCode == 200) {
      final String responseString = response.body;
      print('event track success =====   $responseString');
    } else {
      final String responseString = response.body;
      print('event track failed =====   $responseString');
    }
  }

  static void identify(String userID, AttributeBuilder properties, OtherBuilder otherid) async {
    String serverUrl = FlutterManifest.SERVER_URL;
    String writeKey = FlutterManifest.WRITE_KEY;
    bool isDebugMode = FlutterManifest.DEBUG_MODE;
    bool isEnablePus = FlutterManifest.ENABLE_PUSH;

    GenricModel genricModel = GenricModel();
    genricModel.name = userID;
    genricModel.properties = properties.properties;
    genricModel.otherid = otherid.otherid;
    GenricModel? fMain = await genricModel;
    //After taking data from user.. create our json payload
    //getting device details

    String? deviceId;
    String? deviceType;
    String? deviceManufacture;
    String? deviceModel;
    String? versionName;

    String? appName;
    String? appVersion;
    String? appBuild;

    String? googleAdvertisingID;
    dynamic? googleAdvertisingBool;

    String? networkIP;

    String? appScreenHeightWidth;
    String? appScreenHeightHeight;

    String? appDataReceivedAt;
    String? appDataSenddAt;
    String? appOriginalTimetemp;
    String? appTimestemp;
    String? appServerTs;
    String? appMsgId;

    try {
      deviceId = await _getDeviceId;
      deviceType = await _getPlatFormDeviceType;
      deviceModel = await _getDeviceModel;
      deviceManufacture = await _getManufactureModel;
      versionName = await _getOSVersion;
      appName = await _getAppName;
      appVersion = await _getAppVersion;
      appBuild = await _getBuildNumber;
      googleAdvertisingID = await _getGoogleadvertisingID;
      googleAdvertisingBool = await _getGoogleadvertisingIDBoolValue;
      networkIP = await _getNetworkIP;

      appScreenHeightHeight = await _getScreenHeight;
      appScreenHeightWidth = await _getScreenWidth;

      appDataReceivedAt = await _getReceivedAt;
      appDataSenddAt = await _getSendAt;

      appOriginalTimetemp = await _getOriginalTimestemp;
      appTimestemp = await _getTimestemp;
      appServerTs = await _getServerTs;
      appMsgId = await _getMessageId;
    } on PlatformException {
      deviceId = 'Failed to get information.';
    }

    Map<dynamic, dynamic> lib = {
      "library": {
        "name": "Flutter SDK",
        "version": "0.02",
      },
      "server_ts": appServerTs,
      "app": {
        "name": appName,
        "version": appVersion,
        "build": appBuild,
      },
      "device": {
        "id": deviceId,
        "advertisingId": googleAdvertisingID,
        "adTrackingEnabled": googleAdvertisingBool,
        "manufacturer": deviceManufacture,
        "model": deviceModel,
        "type": deviceType,
        "token": "ff15bc0c20c4aa6cd50854ff165fd265c838e5405bfeb9571066395b8c9da449",
      },
      "screen": {
        "width": appScreenHeightWidth,
        "height": appScreenHeightHeight,
        "density": 2,
      },
      "userAgent": {
        "osType": deviceType,
        "osVersion": versionName,
      },
      "ip": networkIP
    };

    Map map = {
      "id": deviceId,
      "userId": "userId",
      "otherIds": fMain.otherid,
      "event": userID,
      "messageId": appMsgId,
      "customerProperties": fMain.properties,
      "receivedAt": appDataReceivedAt,
      "sentAt": appDataSenddAt,
      "timestamp": appTimestemp,
      "type": "identify",
      "originalTimestamp": appOriginalTimetemp,
      "writeKey": writeKey,
    };
    map.addAll({"context": lib});

    //After the full payload is created..now we send all to lemnisk server
    print('identify event' + serverUrl);
    Uri uri = Uri.parse(serverUrl);
    var response = await http.post(uri, body: JsonEncoder().convert(map));
    if (response.statusCode == 200) {
      final String responseString = response.body;
      print('event identify success =====   $responseString');
    } else {
      final String responseString = response.body;
      print('event identify failed =====   $responseString');
    }
  }

  static void screen(String screenName, AttributeBuilder properties, OtherBuilder otherid) async {
    String serverUrl = FlutterManifest.SERVER_URL;
    String writeKey = FlutterManifest.WRITE_KEY;
    bool isDebugMode = FlutterManifest.DEBUG_MODE;
    bool isEnablePus = FlutterManifest.ENABLE_PUSH;
    //After taking data from user.. create our json payload

    GenricModel genricModel = GenricModel();
    genricModel.name = screenName;
    genricModel.properties = properties.properties;
    genricModel.otherid = otherid.otherid;
    GenricModel? fMain = await genricModel;

    //getting device details

    String? deviceId;
    String? deviceType;
    String? deviceManufacture;
    String? deviceModel;
    String? versionName;

    String? appName;
    String? appVersion;
    String? appBuild;

    String? googleAdvertisingID;
    dynamic? googleAdvertisingBool;

    String? networkIP;

    String? appScreenHeightWidth;
    String? appScreenHeightHeight;

    String? appDataReceivedAt;
    String? appDataSenddAt;
    String? appOriginalTimetemp;
    String? appTimestemp;
    String? appServerTs;
    String? appMsgId;

    try {
      deviceId = await _getDeviceId;
      deviceType = await _getPlatFormDeviceType;
      deviceModel = await _getDeviceModel;
      deviceManufacture = await _getManufactureModel;
      versionName = await _getOSVersion;
      appName = await _getAppName;
      appVersion = await _getAppVersion;
      appBuild = await _getBuildNumber;
      googleAdvertisingID = await _getGoogleadvertisingID;
      googleAdvertisingBool = await _getGoogleadvertisingIDBoolValue;
      networkIP = await _getNetworkIP;

      appScreenHeightHeight = await _getScreenHeight;
      appScreenHeightWidth = await _getScreenWidth;

      appDataReceivedAt = await _getReceivedAt;
      appDataSenddAt = await _getSendAt;

      appOriginalTimetemp = await _getOriginalTimestemp;
      appTimestemp = await _getTimestemp;
      appServerTs = await _getServerTs;
      appMsgId = await _getMessageId;
    } on PlatformException {
      deviceId = 'Failed to get information.';
    }

    Map<dynamic, dynamic> lib = {
      "library": {
        "name": "Flutter SDK",
        "version": "0.02",
      },
      "server_ts": appServerTs,
      "app": {
        "name": appName,
        "version": appVersion,
        "build": appBuild,
      },
      "device": {
        "id": deviceId,
        "advertisingId": googleAdvertisingID,
        "adTrackingEnabled": googleAdvertisingBool,
        "manufacturer": deviceManufacture,
        "model": deviceModel,
        "type": deviceType,
        "token": "ff15bc0c20c4aa6cd50854ff165fd265c838e5405bfeb9571066395b8c9da449",
      },
      "screen": {
        "width": appScreenHeightWidth,
        "height": appScreenHeightHeight,
        "density": 2,
      },
      "userAgent": {
        "osType": deviceType,
        "osVersion": versionName,
      },
      "ip": networkIP
    };

    Map map = {
      "id": deviceId,
      "userId": "userId",
      "otherIds": fMain.otherid,
      "event": screenName,
      "messageId": appMsgId,
      "customerProperties": fMain.properties,
      "receivedAt": appDataReceivedAt,
      "sentAt": appDataSenddAt,
      "timestamp": appTimestemp,
      "type": "screen",
      "originalTimestamp": appOriginalTimetemp,
      "writeKey": writeKey,
    };
    map.addAll({"context": lib});

    //After the full payload is created..now we send all to lemnisk server
    print('Screen event' + serverUrl);
    Uri uri = Uri.parse(serverUrl);
    var response = await http.post(uri, body: JsonEncoder().convert(map));
    if (response.statusCode == 200) {
      final String responseString = response.body;
      print('event Screen success =====   $responseString');
    } else {
      final String responseString = response.body;
      print('event Screen failed =====   $responseString');
    }
  }
}
